#! /usr/local/bin/ruby

require 'xmlparser'
require 'xmltree'
require 'xmltreebuilder'

module XML
  module SimpleTree
    class Node
      def is_text?
	self.is_a? XML::DOM::Text
      end

      def is_element?
	self.is_a? XML::DOM::Element
      end

      def children_to_s
	@children.to_a.collect { |n|
	  n.to_s
	}.join
      end
    end

    class Element
      def text_value1
	return '' if @children.nil? || @children.size == 0
	return @children[0].nodeValue if @children.size == 1
	raise 'complex node'
      end

      def text_value(trim=true)
	return '' if @children.nil?
	str = ''
	@children.each do |c|
	  str += c.nodeValue if c.is_text?
	  str += c.text_value(trim) if c.is_element?
	end
	if trim
	  str.sub!(/^\s*/p, '')
	  str.sub!(/\s*$/p, '')
	end
	str
      end

      def each_element(deep=true, tagname='*')
	@children.each do |c|
	  if c.is_element?
	    yield c if tagname == '*' || c.nodeName == tagname
	    if deep 
	      c.each_element(deep, tagname) do |e|
		yield e
	      end
	    end
	  end
	end if @children
      end

      def getFirstElementByTag(tagname)
	e = nil
	self.each_element(true, tagname) do |e|
	  break
	end
	e
      end
    end
  end
end

if __FILE__ == $0
  def test(tree)
    tree.each_element(true, 'TD') do |e|
      p e.text_value
      p e.children_to_s
    end
  end

  def main(xml)
    begin
      builder = XML::SimpleTreeBuilder.new(1)
      tree = builder.parse(xml, true)
      tree.documentElement.normalize
      test(tree.documentElement)
    rescue XMLParserError
      line = builder.line
      print "#{$0}: #{$!} (line:#{line})\n"
    end
  end

  def xml_sample
    <<EOS
<TABLE>
<TR>
<TH>hello</TH>
<TH>world</TH>
</TR>
<TR>
<TD>Ruby</TD>
<TD>I like <EM>Ruby</EM></TD>
</TR>
</TABLE>
EOS
  end

  main(xml_sample)
end
