#! /usr/local/bin/ruby

require 'domstuff/domstuff'

class DOMTable
  class DOMTableItem
    def initialize(root)
      @hash = {}
      root.each_element(false, '*') do |e|
	@hash[e.nodeName] = e      
      end
    end

    def keys
      @hash.keys
    end

    def text_value(key)
      return nil unless @hash[key]
      @hash[key].text_value
    end

    def children_to_s(key)
      return nil unless @hash[key]
      @hash[key].children_to_s
    end

    def to_xml(key)
      children_to_s(key)
    end

    def node(key)
      @hash[key]
    end

    def [](key)
      to_xml(key)
    end
  end

  def initialize(root, tagname)
    @items = []
    root.each_element(false, tagname) do |e|
      @items.push DOMTableItem.new(e)
    end
  end

  def each
    @items.each do |e|
      yield e
    end
  end

  def size
    @items.size
  end
  alias length size
  
  def to_a
    @items
  end
end

if __FILE__ == $0
  def test(tree)
    list = DOMTable.new(tree, 'ȯ')
    list.each do |item|
      item.keys.each do |k|
	puts "#{k}: #{item[k]}"
      end
      puts 
    end
  end

  def main(f)
    begin
      require 'xmltreebuilder-ja'

      builder = XML::JapaneseTreeBuilder.new(1)
      tree = builder.parseStream(f, true)
      tree.documentElement.normalize
      test(tree.documentElement)
    rescue XMLParserError
      line = builder.line
      print "#{$0}: #{$!} (line:#{line})\n"
    end
  end

  main($<)
end
