#! /usr/local/bin/ruby

require 'domstuff/domstuff'

module XML
  module SimpleTree
    class Node
      def visited(v=true)
	@visited = v
      end
      def was_visited?
	@visited
      end

      def clear_visited_flag
	visited(false)
	@children.each do |e|
	  e.clear_visited_flag
	end if @children
      end

      def visit
	return if was_visited?
	yield self
	@children.each do |e|
	  e.visit do |c|
	    yield c
	  end
	end if @children
	visited
      end
    end
  end
end

if __FILE__ == $0
  def test(tree)
    tree.documentElement.each_element(true, 'TD') do |e|
      p e
      if e.nodeName == 'TD'
	e.visited
      end
    end
    tree.documentElement.visit do |e|
      if e.is_element?
	p e
      end
    end
  end

  def main(xml)
    begin
      builder = XML::SimpleTreeBuilder.new(1)
      tree = builder.parse(xml, true)
      tree.documentElement.normalize
      test(tree)
    rescue XMLParserError
      line = builder.line
      print "#{$0}: #{$!} (line:#{line})\n"
    end
  end

  def xml_sample
    <<EOS
<TABLE>
<TR>
<TH>hello</TH>
<TH>world</TH>
</TR>
<TR>
<TD>Ruby</TD>
<TD>I like <EM>Ruby</EM></TD>
</TR>
</TABLE>
EOS
  end

  main(xml_sample)
end
